from typing import Dict, List

from sampledbapi import getData

__all__ = ["getList", "get"]


def getList() -> List:
    """Get a list of all locations."""
    return getData("locations")


def get(location_id: int) -> Dict:
    """Get the specific location (location_id)."""
    if isinstance(location_id, int):
        return getData("locations/{}".format(location_id))
    else:
        raise TypeError()



from typing import Dict, List

from sampledbapi import getData

__all__ = ["getList", "get", "getCurrent"]


def getList() -> List:
    """Get a list of all users."""
    return getData("users")


def get(user_id: int) -> Dict:
    """Get the specific user (user_id)."""
    if isinstance(user_id, int):
        return getData("users/{}".format(user_id))
    else:
        raise TypeError()


def getCurrent() -> Dict:
    """Get the current user."""
    return getData("users/me")



import base64
import os
from io import IOBase
from typing import BinaryIO, Dict, List

from requests import Response

from sampledbapi import getData, postData, putData

__all__ = ["getList", "get", "getVersion", "getPublic",
           "getAllUsersPermissions", "getUserPermissions",
           "getAllGroupsPermissions", "getGroupPermissions",
           "getAllProjectsPermissions", "getProjectPermissions",
           "getLocationList", "getLocation", "getFileList", "getFileInfo"]


def getList() -> List:
    """Get a list of all objects visible to the current user.

    The list only contains the current version of each object. By passing the
    parameter q to the query, the Advanced Search can be used. By passing the
    parameters action_id or action_type objects can be filtered by the action
    they were created with or by their type (e.g. sample or measurement).
    """
    return getData("objects")


def get(object_id: int) -> Dict:
    """Get the current version of an object (object_id)."""
    if isinstance(object_id, int):
        return getData("objects/{}".format(object_id))
    else:
        raise TypeError()


def getVersion(object_id: int, version_id: int) -> Dict:
    """Get the specific version (version_id) of an object (object_id)."""
    if isinstance(object_id, int) and isinstance(version_id, int):
        return getData(
            "objects/{}/versions/{}" % (object_id, version_id)
        )
    else:
        raise TypeError()


"""Permissions"""


def getPublic(object_id: int) -> bool:
    """Get whether or not an object is public."""
    if isinstance(object_id, int):
        return getData("objects/{}/permissions/public".format(object_id))
    else:
        raise TypeError()


def setPublic(object_id: int, public: bool) -> Response:
    """Set whether or not an object is public."""
    if isinstance(object_id, int) and isinstance(public, bool):
        return putData("objects/{}/permissions/public".format(object_id),
                       public)
    else:
        raise TypeError()


def getAllUsersPermissions(object_id: int) -> Dict:
    """Get a mapping of user IDs to their permissions."""
    if isinstance(object_id, int):
        return getData("objects/{}/permissions/users".format(object_id))
    else:
        raise TypeError()


def getUserPermissions(object_id: int, user_id: int) -> str:
    """Get the permissions of a user for an object."""
    if isinstance(object_id, int) and isinstance(user_id, int):
        return getData(
            "objects/{}/permissions/users/{}".format(object_id, user_id)
        )
    else:
        raise TypeError()


def setUserPermissions(object_id: int, user_id: int,
                       permissions: str) -> Response:
    """Set the permissions of a user for an object."""
    if (isinstance(object_id, int) and isinstance(user_id, int) and
            isinstance(permissions, str)):
        return putData(
            "objects/{}/permissions/users/{}".format(object_id, user_id),
            permissions
        )
    else:
        raise TypeError()


def getAllGroupsPermissions(object_id: int) -> Dict:
    """Get a mapping of basic group IDs to their permissions."""
    if isinstance(object_id, int):
        return getData("objects/{}/permissions/groups".format(object_id))
    else:
        raise TypeError()


def getGroupPermissions(object_id: int, group_id: int) -> str:
    """Get the permissions of a basic group for an object."""
    if isinstance(object_id, int) and isinstance(group_id, int):
        return getData(
            "objects/{}/permissions/groups/{}" % (object_id, group_id)
        )
    else:
        raise TypeError()


def setGroupPermissions(object_id: int, group_id: int,
                        permissions: str) -> Response:
    """Set the permissions of a basic group for an object."""
    if (isinstance(object_id, int) and isinstance(group_id, int) and
            isinstance(permissions, str)):
        return putData(
            "objects/{}/permissions/groups/{}".format(object_id, group_id),
            permissions
        )
    else:
        raise TypeError()


def getAllProjectGroupsPermissions(object_id: int) -> Dict:
    """Get a mapping of project group IDs to their permissions."""
    if isinstance(object_id, int):
        return getData("objects/{}/permissions/projects".format(object_id))
    else:
        raise TypeError()


def getProjectGroupPermissions(object_id: int, project_id: int) -> str:
    """Get the permissions of a project group for an object."""
    if isinstance(object_id, int) and isinstance(project_id, int):
        return getData(
            "objects/{}/permissions/projects/{}" % (object_id, project_id)
        )
    else:
        raise TypeError()


def setProjectGroupPermissions(object_id: int, project_id: int,
                               permissions: str) -> Response:
    """Set the permissions of a project group for an object."""
    if (isinstance(object_id, int) and isinstance(project_id, int) and
            isinstance(permissions, str)):
        return putData(
            "objects/{}/permissions/projects/{}".format(object_id, project_id),
            permissions
        )
    else:
        raise TypeError()


"""Locations"""


def getLocationList(object_id: int) -> List:
    """Get a list of all object locations assignments for a specific object (object_id)."""
    if isinstance(object_id, int):
        return getData("objects/{}/locations".format(object_id))
    else:
        raise TypeError()


def getLocation(object_id: int, location_id: int) -> Dict:
    """Get a specific object location assignment (location_id) for a specific object (object_id)."""
    if isinstance(object_id, int) and isinstance(location_id, int):
        return getData(
            "objects/{}/locations/{}" % (object_id, location_id)
        )
    else:
        raise TypeError()


"""Files"""


def getFileList(object_id: int) -> List:
    """Get a list of all files for a specific object (object_id)."""
    if isinstance(object_id, int):
        return getData("objects/{}/files".format(object_id))
    else:
        raise TypeError()


def getFileInfo(object_id: int, file_id: int) -> Dict:
    """Get a specific file (file_id) for a specific object (object_id)."""
    if isinstance(object_id, int) and isinstance(file_id, int):
        return getData(
            "objects/{}/files/{}" % (object_id, file_id)
        )
    else:
        raise TypeError()


def uploadFile(object_id: int, path: str) -> Response:
    """Create a new file with local storage for a specific object (object_id).

    Args:
        object_id (int): ID of the object
        path (str): Path of the file to be uploaded.
    """
    if isinstance(object_id, int) and isinstance(path, str):
        with open(path, "rb") as f:
            r = uploadFileRaw(object_id, os.path.basename(path), f)
        return r
    else:
        raise TypeError()


def uploadFileRaw(object_id: int, name: str, file_obj: BinaryIO) -> Response:
    """Create a new file with local storage for a specific object (object_id).

    Args:
        object_id (int): ID of the object
        name (str): Name that the file will have online.
        file_obj (BinaryIO): A binary stream that can be read to be uploaded.
    """
    if (isinstance(object_id, int) and isinstance(name, str) and
            isinstance(file_obj, IOBase)):
        base64encoded = base64.b64encode(file_obj.read())
        return postData("objects/{}/files/".format(object_id),
                        {"storage": "local", "original_file_name": name,
                         "base64_content": base64encoded.decode()})
    else:
        raise TypeError()


def postLink(object_id: int, url: str) -> Response:
    """Create a new file with url storage for a specific object (object_id)."""
    if isinstance(object_id, int) and isinstance(url, str):
        return postData("objects/{}/files/".format(object_id),
                        {"storage": "url", "url": url})
    else:
        raise TypeError()


"""Comments"""


def getCommentList(object_id: int) -> List:
    """Get a list of all comments for a specific object (object_id)."""
    if isinstance(object_id, int):
        return getData("objects/{}/comments".format(object_id))
    else:
        raise TypeError()


def getComment(object_id: int, comment_id: int) -> Dict:
    """Get specific comment (comment_id) for a specific object (object_id)."""
    if isinstance(object_id, int) and isinstance(comment_id, int):
        return getData("objects/{}/comments/{}".format(object_id, comment_id))
    else:
        raise TypeError()


def postComment(object_id: int, comment: str) -> Response:
    """Create a new comment for a specific object (object_id)."""
    if isinstance(object_id, int) and isinstance(comment, str):
        return postData("objects/{}/comments/".format(object_id),
                        {"content": comment})
    else:
        raise TypeError()



from typing import Dict, List

from sampledbapi import getData

__all__ = ["getList", "get", "getLogEntryList", "getLogEntry",
           "getLogCategoryList", "getLogCategory",
           "getFileAttachmentList", "getFileAttachment",
           "getObjectAttachmentList", "getObjectAttachment"]


def getList() -> List:
    """Get a list of all instruments."""
    return getData("instruments")


def get(instrument_id: int) -> Dict:
    """Get the specific instrument (instrument_id)."""
    if isinstance(instrument_id, int):
        return getData("instruments/{}".format(instrument_id))
    else:
        raise TypeError()


"""Instrument log entries"""


def getLogEntryList(instrument_id: int) -> List:
    """Get a list of all log entries for a specific instrument (instrument_id)."""
    if isinstance(instrument_id, int):
        return getData("instruments/{}/log_entries".format(instrument_id))
    else:
        raise TypeError()


def getLogEntry(instrument_id: int, log_entry_id: int) -> Dict:
    """Get the specific log entry (log_entry_id) for an instrument (instrument_id)."""
    if isinstance(instrument_id, int) and isinstance(log_entry_id, int):
        return getData("instruments/{}/log_entries/{}".format(
            instrument_id, log_entry_id))
    else:
        raise TypeError()


def getLogCategoryList(instrument_id: int) -> List:
    """Get a list of all log category for a specific instrument (instrument_id)."""
    if isinstance(instrument_id, int):
        return getData("instruments/{}/log_categories".format(instrument_id))
    else:
        raise TypeError()


def getLogCategory(instrument_id: int, log_category_id: int) -> Dict:
    """Get the specific log categories (log_category_id) for an instrument (instrument_id)."""
    if isinstance(instrument_id, int) and isinstance(log_category_id, int):
        return getData("instruments/{}/log_categories/{}".format(
            instrument_id, log_category_id))
    else:
        raise TypeError()


def getFileAttachmentList(instrument_id: int, log_entry_id: int) -> List:
    """Get a list of file attachments for a specific log entry (log_entry_id) for an instrument (instrument_id)."""
    if isinstance(instrument_id, int) and isinstance(log_entry_id, int):
        return getData("instruments/{}/log_entries/{}/file_attachments".format(
            instrument_id, log_entry_id))
    else:
        raise TypeError()


def getFileAttachment(instrument_id: int, log_entry_id: int, file_attachment_id: int) -> Dict:
    """Get a specific file attachment (file_attachment_id) for a log entry (log_entry_id) for an instrument (instrument_id)."""
    if (isinstance(instrument_id, int) and isinstance(log_entry_id, int) and
            isinstance(file_attachment_id, int)):
        return getData("instruments/{}/log_entries/{}/file_attachments/{}".format(
            instrument_id, log_entry_id, file_attachment_id))
    else:
        raise TypeError()


def getObjectAttachmentList(instrument_id: int, log_entry_id: int) -> List:
    """Get a list of object attachments for a specific log entry (log_entry_id) for an instrument (instrument_id)."""
    if isinstance(instrument_id, int) and isinstance(log_entry_id, int):
        return getData("instruments/{}/log_entries/{}/object_attachments".format(
            instrument_id, log_entry_id))
    else:
        raise TypeError()


def getObjectAttachment(instrument_id: int, log_entry_id: int, object_attachment_id: int) -> Dict:
    """Get a specific file attachment (file_attachment_id) for a log entry (log_entry_id) for an instrument (instrument_id)."""
    if isinstance(instrument_id, int) and isinstance(log_entry_id, int) and isinstance(object_attachment_id, int):
        return getData("instruments/{}/log_entries/{}/object_attachments/{}".format(
            instrument_id, log_entry_id, object_attachment_id))
    else:
        raise TypeError()



from typing import Dict, List

from sampledbapi import getData

__all__ = ["getList", "get"]


def getList() -> List:
    """Get a list of all action types."""
    return getData("action_types")


def get(type_id: int) -> Dict:
    """Get the specific action type (type_id)."""
    if isinstance(type_id, int):
        return getData("actions/{}".format(type_id))
    else:
        raise TypeError()



import json
from typing import Dict, List

import requests

__all__ = ["authenticate", "actions", "actiontypes", "instruments",
           "locations", "objects", "users"]

_address = None
_api_key = None


def authenticate(address: str, api_key: str):
    global _address, _api_key
    _address = address
    _api_key = api_key


def __headers() -> Dict:
    return {"Authorization": "Bearer " + _api_key}


def getData(path: str) -> str:
    address = _address + "/api/v1/" + path
    r = requests.get(address, headers=__headers())
    try:
        return json.loads(r.text)
    except Exception as e:
        # We need better error handling here
        raise Exception("JSON could not be parsed: " + str(e) +
                        "\nJSON was\n" + r.text)


def postData(path: str, data: str) -> requests.Response:
    address = _address + "/api/v1/" + path
    data = json.dumps(data)
    return requests.post(address, headers=__headers(), data=data)


def putData(path: str, data: str) -> requests.Response:
    address = _address + "/api/v1/" + path
    data = json.dumps(data)
    return requests.put(address, headers=__headers(), data=data)



"""
This is the module docstring.
"""

from typing import Dict, List

from sampledbapi import getData

__all__ = ["getList", "get"]


def getList() -> List:
    """Get a list of all actions."""
    return getData("actions")


def get(action_id: int) -> Dict:
    """Get the specific action (action_id)."""
    if isinstance(action_id, int):
        return getData("actions/{}".format(action_id))
    else:
        raise TypeError()
